## Copyright (C) 2019 Pantxo Diribarne
##
## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <https://www.gnu.org/licenses/>.

## -*- texinfo -*-
## @deftypefn {} {@var{retval} =} dump_class_file (@var{class_name}, @var{h}, @var{fname})
##
## @seealso{}
## @end deftypefn

## Author: Pantxo Diribarne <pantxo@dibona>
## Created: 2019-06-22

function retval = dump_class_file (cname, h, fname)
  if (! ischar (cname) || ! isvector (cname) || ! ishghandle (h)
     || ! ischar (fname) || ! isvector (fname))
    print_usage ()
  endif

  fid = fopen (fname, "w+");
  if (fid < 0)
    error ("dump_class_file: unable to open file %s for writing", fname);
  endif
  
  unwind_protect    
    str = __get_classdef__ (cname, h);
    fprintf (fid, "%s", str);
  unwind_protect_cleanup
    fclose (fid);
  end_unwind_protect
  
endfunction

function str = __get_classdef__ (cname, h)
  obj = get (h);
  getprops = fieldnames (obj);
  setprops = fieldnames (obj);
  idx = cellfun (@(str) ! any (strcmp (str, setprops)), getprops);
  getprops = getprops (idx);
  
  str = sprintf ("classdef %s < graphics_placeholder", cname);
  indent = 1;
  ## Properties
  str = __put_line___ ("properties (GetAccess = public, SetAccess = public)",
                       str, indent++, true);
  for ii = 1:numel (setprops)
    str = __put_line___ ([setprops{ii} ";"],str, indent);
  endfor
  str = __put_line___ ("endproperties", str, --indent);
  
  str = __put_line___ ("properties (GetAccess = public, SetAccess = private)",
                       str, indent++, true);
  for ii = 1:numel (getprops)
    str = __put_line___ ([getprops{ii} ";"],str, indent);
  endfor
  str = __put_line___ ("endproperties", str, --indent, false);
  
  str = __put_line___ ("properties (GetAccess = private, SetAccess = private)",
                       str, indent++, true);
  str = __put_line___ ("__handle__;" ,str, indent);
  str = __put_line___ ("endproperties", str, --indent);

  ## Methods
  str = __put_line___ ("methods (Access = public)",
                       str, indent++, true);

  ## Constructor
  str = __put_line___ (sprintf ("function this = %s (h)", cname), ...
                       str, indent++, true);
  str = __put_line___ ("this = this@graphics_placeholder(h);",str, indent);
  str = __put_line___ ("endfunction", str, --indent);

  str = __put_line___ ("endmethods", str, --indent);  
  str = __put_line___ ("endclassdef", str, --indent);             
  
endfunction
  
function str = __put_line___ (l, str, indent, double_newline = false)
  pad = repmat ("  ", 1, indent);
  nl = ifelse (double_newline, "\n\n", "\n");
  str = [str, nl, pad, l];
endfunction
