classdef graphics_placeholder < handle

  properties (GetAccess = protected, SetAccess = protected)
    __hdl__= [];
    __plist__= {};
  endproperties
  
  methods (Access = public)

    function this = graphics_placeholder (h = [], caller = "")
      this.__hdl__ = h;
      if (! isempty (h))
        this.__plist__ = tolower (fieldnames (get (h)));
      endif
    endfunction

    function h = get_handle (this)
      h = this.__hdl__;
    endfunction

    function update_plist (this)
      this.__plist__ = tolower (builtin ("properties", this));
    endfunction
    
    function sref = subsref (this, s)
      switch (s(1).type)
        case "."
          switch (tolower (s(1).subs))
            case this.__plist__
              sref = this.get (s(1).subs);
            otherwise
              error ("%s: unknown property '%s'", class (this), s(1).subs);
          endswitch
        case "()"
          error ("%s: cannot be index with ()", class (this));
      endswitch
    endfunction

    function val = get (this, varargin)
      val = get (this.__hdl__, varargin{:});
      if (ishghandle (val))
        val = this.__hg_object__ (val);
      endif
    endfunction

    function this = subsasgn (this, s, val)
      switch (s(1).type)
        case "."
          switch (tolower (s(1).subs))
            case this.__plist__
              set (this.__hdl__, s(1).subs, val);
            otherwise
              error ("%s: unknown property '%s'", class (this), s(1).subs);
          endswitch
        case "()"
          error ("%s: cannot be index with ()", class (this));
      endswitch
    endfunction

    function varargout = set (this, prop, varargin)
      if (numel (varargin) == 0)
        if (nargout == 0)
          set (this.__hdl__, prop);
        else
          varargout{1} = set (this.__hdl__, prop);
        endif
      else
        set (this.__hdl__, prop, varargin{:});
      endif
    endfunction
    
    function disp (this)
      printf ("  %s object with properties:\n\n", class (this));
      plist = builtin ("properties", this);
      vals = get (this.__hdl__, plist);
      plist = strjust (char (plist), "right");
      typ = get (this.__hdl__, "type"); 
      
      
      for ii = 1:rows (plist)
        prop = strtrim (plist(ii,:));
        if (strcmpi (prop, "type"))
          continue;
        endif
        try
          defval = get (groot, ["default" typ prop]);
          do_display = ! isequal (vals{ii}, defval);
        catch
          do_display = true;
        end_try_catch
        if (do_display)
          printf ("    %s : %s\n", plist(ii,:), dispval (this, vals{ii}));
        endif
      endfor
    endfunction
  endmethods

  methods (Access = private)
    function str = dispval (this, val)
      str = strtrim (disp (val));
      if (rows (val) > 1)
        str = sprintf ("%d-by-%d %s", rows (val), columns (val), ...
                       class (val));
      elseif (strcmp (class (val), "char"))
        if (columns (val) > 40)
          str = [val(1:40) " ..."];
        endif
      elseif (columns (val) > 4)
        str = sprintf ("%d-by-%d %s", rows (val), columns (val), ...
                       class (val));
      endif
    endfunction
    
    function obj = __hg_object__ (this, h)
      switch get (h, "type")
        case "figure"
          obj = __figure__ (h);
        case "axes"
          obj = __axes__ (h);
        case "image"
          obj = __image__ (h);
        case "patch"
          obj = __patch__ (h);
        case "line"
          obj = __line__ (h);
        otherwise
          obj = h;
      endswitch
    endfunction
  endmethods
endclassdef

